//+------------------------------------------------------------------+
//| Frame test example                                               |
//+------------------------------------------------------------------+
#define SIZE_X 500
#define SIZE_Y 100
input int Line     = 0;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   return;
//---
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
  }
//+------------------------------------------------------------------+
//| CWavelet                                                         |
//+------------------------------------------------------------------+
class CWavelet
  {
protected:
   int               m_xsize;
   int               m_ysize;
   int               m_x0;
   int               m_y0;
   int               m_maxcolor;
   string            m_res_name;
   string            m_label_name;
   uint              m_buf[];
   double            m_global_min;
   double            m_global_max;
   uchar             m_palette[3*320];
   double            m_wavelet_data[];
   uint              m_bmp_buffer[];
public:
   int               GetPalColor(int index);
   void              Blend(uint c1,uint c2,uint r1,uint g1,uint b1,uint r2,uint g2,uint b2);
   void              Create(string name,int x_size,int y_size,color col);
   double            Morlet(double t);
   double            W(double a,double b,double  &f[],int n,bool normneeded);
   void              CalculateWaveletLine(double &data[],int datacount,double a,double b1,double b2,int xsize,bool normneeded,int line_index,double &WaveletDataLine[]);
   void              AddWaveletLine(long line_index,double &WaveletDataLine[]);
   void              ShowImage();
  };
//+------------------------------------------------------------------+
//| GetPalColor                                                      |
//+------------------------------------------------------------------+
int CWavelet::GetPalColor(int index)
  {
   int ind=index;
   if(ind<=0) {ind=0;}
   if(ind>255) {ind=255;}
   uchar r=m_palette[3*(ind)];
   uchar g=m_palette[3*(ind)+1];
   uchar b=m_palette[3*(ind)+2];
   return(b+256*g+65536*r);
  }
//+------------------------------------------------------------------+
//| Gradient palette                                                 |
//+------------------------------------------------------------------+
void CWavelet::Blend(uint c1,uint c2,uint r1,uint g1,uint b1,uint r2,uint g2,uint b2)
  {
   int n=int(c2-c1);
   for(int i=0; i<=n; i++)
     {
      if((c1+i+2)<ArraySize(m_palette))
        {
         m_palette[3*(c1+i)]=uchar(MathRound(1*(r1*(n-i)+r2*i)*1.0/n));
         m_palette[3*(c1+i)+1]=uchar(MathRound(1*(g1*(n-i) + g2*i) * 1.0/ n));
         m_palette[3*(c1+i)+2]=uchar(MathRound(1*(b1*(n-i) + b2*i) * 1.0/ n));
        }
     }
  }
//+------------------------------------------------------------------+
//| Morlet wavelet function                                          |
//+------------------------------------------------------------------+
double CWavelet::Morlet(double t)
  {
   double t2=t*t;
   double f=MathExp(-t2/2)*MathCos(2*3.1415926535897932384626*t);
   return(f);
  }
//+------------------------------------------------------------------+
//| Amplitude wavelet function                                       |
//+------------------------------------------------------------------+
double CWavelet::W(double a,double b,double  &f[],int n,bool normneeded)
  {
//double B=2.0; //Mexican hat
   double B=1.0; //Morlet
   double B_inv=1.0/B;
   double a_inv=1/a;
   double dt=1;
   double coef=0.0;
   if(!normneeded)
     {
      coef=MathSqrt(a_inv);
     }
   else
     {
      //--- n(a,b) (42)
      for(int k=0; k<n; k++)
        {
         double tk=dt*k;
         double arg=(tk-b)*a_inv;
         arg=-B_inv*arg*arg;
         coef=coef+MathExp(arg);
        }
     }
//--- WA(a,b) (41)
   double sum=0.0;
   for(int k=0; k<n; k++)
     {
      double tk=dt*k;
      sum=sum+f[k]*Morlet((tk-b)*a_inv);
     }
   sum=sum/coef;
   return(sum);
  }
//+------------------------------------------------------------------+
//| CalculateWaveletLine                                             |
//+------------------------------------------------------------------+
void CWavelet::CalculateWaveletLine(double &data[],int datacount,double a,double b1,double b2,int xsize,bool normneeded,int line_index,double &wavelet_data_line[])
  {
   ArrayResize(wavelet_data_line,xsize);
   double b=b1;
   double db=(b2-b1)/(xsize);
   int wdata_count=0;
   for(int i=0; i<xsize; i++)
     {
      wavelet_data_line[wdata_count++]=W(a,b1+i*db,data,datacount,normneeded);
     }
  }
//+------------------------------------------------------------------+
//| Create                                                           |
//+------------------------------------------------------------------+
void CWavelet::Create(string name,int x_size,int y_size,color col)
  {
   m_xsize=x_size;
   m_ysize=y_size;
   ArrayResize(m_bmp_buffer,m_xsize*m_ysize);
   ArrayResize(m_wavelet_data,m_xsize*m_ysize);
   m_maxcolor=320;
   m_res_name=name;
   m_label_name=m_res_name;
   StringToUpper(m_label_name);
   ResourceCreate(m_res_name,m_bmp_buffer,m_xsize,m_ysize,0,0,0,COLOR_FORMAT_XRGB_NOALPHA);
   ObjectCreate(0,m_label_name,OBJ_BITMAP_LABEL,0,0,0);
   ObjectSetInteger(0,m_label_name,OBJPROP_XDISTANCE,20);
   ObjectSetInteger(0,m_label_name,OBJPROP_YDISTANCE,20);
   ObjectSetString(0,m_label_name,OBJPROP_BMPFILE,NULL);
   ObjectSetString(0,m_label_name,OBJPROP_BMPFILE,"::"+m_label_name);
   Blend(0,160,170,0,0,255,255,255);
   Blend(161,319,255,255,255,0,170,0);
  }
//+------------------------------------------------------------------+
//| AddWaveletLine                                                   |
//+------------------------------------------------------------------+
void CWavelet::AddWaveletLine(long line_index,double &WaveletDataLine[])
  {
   if(ArraySize(WaveletDataLine)!=m_xsize || (line_index>=m_ysize))
     {
      Print("Problem with line ",line_index);
      return;
     }
   double local_min=WaveletDataLine[0];
   double local_max=WaveletDataLine[0];
   for(int i=1; i<ArraySize(WaveletDataLine); i++)
     {
      local_min=MathMin(local_min,WaveletDataLine[i]);
      local_max=MathMax(local_max,WaveletDataLine[i]);
     }
   if(line_index==0)
     {
      m_global_max=local_max;
      m_global_min=local_min;
     }
   else
     {
      m_global_max=MathMax(m_global_max,local_max);
      m_global_min=MathMin(m_global_min,local_min);
     }

   for(int i=0; i<ArraySize(WaveletDataLine); i++)
     {
      m_wavelet_data[int(line_index*m_xsize+i)]=WaveletDataLine[i];
     }
//--- add line to buffer
   double range=m_global_max-m_global_min;
   if(range>0)
      for(int i=0; i<m_xsize; i++)
        {
         int pos=int(line_index*m_xsize+i);
         int colindex=int(m_maxcolor*(m_wavelet_data[pos]-m_global_min)/range);
         m_bmp_buffer[pos]=GetPalColor(colindex);
        }
  }
//+------------------------------------------------------------------+
//| ShowImage                                                        |
//+------------------------------------------------------------------+
void CWavelet::ShowImage()
  {
   ResourceCreate(m_res_name,m_bmp_buffer,m_xsize,m_ysize,0,0,0,COLOR_FORMAT_XRGB_NOALPHA);
   ChartRedraw();
  }

#include <ClassProgressBar.mqh>  
CProgressBar progress;
CWavelet wavelet;
//+------------------------------------------------------------------+
//| OnTesterInit                                                     |
//+------------------------------------------------------------------+
void OnTesterInit()
  {
   wavelet.Create("Wavelet",SIZE_X,SIZE_Y,clrWhite);
   progress.Create(0,"Calculation",0,150,20);
   progress.Minimum=0;
   progress.Maximum=100;
   progress.Text("Calculation");
  }
//+------------------------------------------------------------------+
//| OnTesterPass                                                     |
//+------------------------------------------------------------------+
void OnTesterPass()
  {
   string  name;
   ulong   pass;
   long    id;
   double  val,WaveletLine[];
   static int passes_ok=0;
   int     maxpass=SIZE_Y;

//---
   while(FrameNext(pass,name,id,val,WaveletLine))
     {
      long line_index=id;
      Print("pass=",pass," id=",id," val=",val);

      wavelet.AddWaveletLine(line_index,WaveletLine);
      wavelet.ShowImage();

      passes_ok++;
      progress.Value(int(passes_ok*100.0/maxpass));
     }
  }
//+------------------------------------------------------------------+
//| OnTesterDeinit                                                   |
//+------------------------------------------------------------------+
void OnTesterDeinit()
  {
//---show final image
   wavelet.ShowImage();
  }
//+------------------------------------------------------------------+
//| Weirstrass function                                              |
//+------------------------------------------------------------------+
double w(double x,double a,double b)
  {
   double sum=0.0;
   double b0=b;
   double a0=a;
   for(int n=0; n<35; n++)
     {
      double v=b0*MathCos(a0*3.1415926535897932384626*x);
      sum=sum+v;
      a0=a0*a;
      b0=b0*b;
     }
   return(sum);
  }
//+------------------------------------------------------------------+
//| Tester function                                                  |
//+------------------------------------------------------------------+
double OnTester()
  {
   if(Line>=SIZE_Y)
     {
      Print("Line>=SIZE_Y");
      return(0);
     }
//--- Weirstrass function
   double data[];
   int datacount=SIZE_X;
   ArrayResize(data,datacount);
   double x1=0;
   double x2=2;
   double dx=(x2-x1)/datacount;
   for(int i=0; i<datacount; i++) data[i]=w(x1+dx*i,3,0.65);

//--- wavelet ranges
   double a1=2;
   double a2=120;
   double da=(a2-a1)/SIZE_Y;

   double wavelet_line[];
   int line_index=Line;

   wavelet.CalculateWaveletLine(data,datacount,a1+line_index*da,0,datacount,SIZE_X,true,Line,wavelet_line);

   if(!FrameAdd("x",line_index,0,wavelet_line))
      Print("frame add error: ",GetLastError());
   else
     {
      Print("frame add ok");
     }
//---
   return(0);
  }
//+------------------------------------------------------------------+
